<?php
/**
 * Plugin Name: Revealer
 * Plugin URI: https://1.envato.market/revealer
 * Description: Most effective way to protect your online content from being copy.
 * Version: 2.0.2
 * Author: پرشین اسکریپت
 * Author URI: https://www.persianscript.ir
 * Requires PHP: 5.6
 * Requires at least: 4.0
 * Tested up to: 5.3.2
 **/

/**
 * Revealer - Adds a stylish hover pop-up with five animation effects for internal WordPress website links.
 * Exclusively on Envato Market: https://1.envato.market/revealer
 *
 * @encoding        UTF-8
 * @version         2.0.2
 * @copyright       Copyright (C) 2018 - 2020 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove;

use Merkulove\Revealer\AssignmentsTab;
use Merkulove\Revealer\PluginUpdater;
use Merkulove\Revealer\Helper;
use Merkulove\Revealer\PluginHelper;
use Merkulove\Revealer\Settings;
use Merkulove\Revealer\EnvatoItem;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/** Include plugin autoloader for additional classes. */
require __DIR__ . '/src/autoload.php';


/**
 * SINGLETON: Core class used to implement a Revealer plugin.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * @since 1.0.0
 **/
final class Revealer {

	/**
	 * Plugin version.
	 *
	 * @string version
	 * @since 2.0.0
	 **/
	public static $version = '';

	/**
	 * Use minified libraries if SCRIPT_DEBUG is turned off.
	 *
	 * @since 1.0.0
	 **/
	public static $suffix = '';

	/**
	 * URL (with trailing slash) to plugin folder.
	 *
	 * @since 1.0.0
	 * @var string
	 **/
	public static $url = '';

	/**
	 * PATH to plugin folder.
	 *
	 * @since 1.0.0
	 * @var string
	 **/
	public static $path = '';

	/**
	 * Plugin base name.
	 *
	 * @since 1.0.0
	 * @var string
	 **/
	public static $basename = '';

	/**
	 * The one true Revealer.
	 *
	 * @since 1.0.0
	 * @var Revealer
	 **/
	private static $instance;

	/**
	 * Sets up a new plugin instance.
	 *
	 * @since  1.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Initialize main variables. */
		$this->initialization();

		/** Define admin hooks. */
		$this->admin_hooks();

		/** Define public hooks. */
		$this->public_hooks();

		/** Define hooks that runs on both the front-end as well as the dashboard. */
		$this->both_hooks();

	}

	/**
	 * Register all of the hooks related to the public-facing functionality.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
	private function public_hooks() {

		/** Add plugin styles. */
		add_action( 'wp_enqueue_scripts', [ $this, 'styles' ] );

		/** Add plugin script. */
		add_action( 'wp_enqueue_scripts', [ $this, 'scripts' ] );

		/** Ajax on the Viewer-Facing Side. */
		add_action( 'wp_ajax_get_page_by_url', [ $this, 'get_page_by_url' ] );
		add_action( 'wp_ajax_nopriv_get_page_by_url', [ $this, 'get_page_by_url' ] );

		/** Allow DIVI in AJAX. */
		add_filter( 'et_builder_load_actions', [ $this, 'divi_builder_load_actions' ] );

	}

	/**
	 * Register all of the hooks related to the admin area functionality.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
	private function admin_hooks() {

		/** Add plugin settings page. */
		Settings::get_instance()->add_settings_page();

		/** Load JS and CSS for Backend Area. */
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_styles' ], 100 ); // CSS.
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_scripts' ], 100 ); // JS.

		/** Remove "Thank you for creating with WordPress" and WP version only from plugin settings page. */
		add_action( 'admin_enqueue_scripts', [$this, 'remove_wp_copyrights'] );

		/** Remove all "third-party" notices from plugin settings page. */
		add_action( 'in_admin_header', [$this, 'remove_all_notices'], 1000 );

	}

	/**
	 * Define hooks that runs on both the front-end as well as the dashboard.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
	private function both_hooks() {

		/** Load translation. */
		add_action( 'plugins_loaded', [$this, 'load_textdomain'] );

	}

	/**
	 * Remove all other notices.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function remove_all_notices() {

		/** Work only on plugin settings page. */
		$screen = get_current_screen();
		if ( $screen->base !== 'toplevel_page_mdp_revealer_settings' ) { return; }

		/** Remove other notices. */
		remove_all_actions( 'admin_notices' );
		remove_all_actions( 'all_admin_notices' );

	}

	/**
	 * Remove "Thank you for creating with WordPress" and WP version only from plugin settings page.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
	public function remove_wp_copyrights() {

		/** Remove "Thank you for creating with WordPress" and WP version from plugin settings page. */
		$screen = get_current_screen(); // Get current screen.

		/** Revealer Settings Page. */
		if ( $screen->base == 'toplevel_page_mdp_revealer_settings' ) {
			add_filter( 'admin_footer_text', '__return_empty_string', 11 );
			add_filter( 'update_footer', '__return_empty_string', 11 );
		}

	}

	/**
	 * Add plugin styles.
	 *
	 * @since   1.0.0
	 * @return void
	 **/
	public function styles() {

		/** CSS. */
		$bg_color   = Settings::get_instance()->options['bgcolor'];
		$text_color = Settings::get_instance()->options['text_color'];
		$css        = "
                .tippy-tooltip,
                .tippy-tooltip .tippy-backdrop {
                    background-color: {$bg_color} !important;
                    color: {$text_color};
                }
                
                .tippy-tooltip #mdp-preloader circle { fill: {$text_color}; }
                .tippy-tooltip #mdp-preloader stop { stop-color: {$text_color}; }

                .tippy-tooltip h3,
                .tippy-tooltip .mdp-excerpt,
                .tippy-tooltip .mdp-excerpt p { color: {$text_color}; }
                    
                .tippy-tooltip[x-placement^='top'] .tippy-arrow { border-top-color: {$bg_color}; }
                .tippy-tooltip[x-placement^='bottom'] .tippy-arrow { border-bottom-color: {$bg_color}; }
                .tippy-tooltip[x-placement^='left'] .tippy-arrow { border-left-color: {$bg_color}; }
                .tippy-tooltip[x-placement^='right'] .tippy-arrow { border-right-color: {$bg_color}; }    
                .tippy-roundarrow path { fill: {$bg_color}; }
            ";

		wp_enqueue_style( 'mdp-revealer', self::$url . 'css/revealer' . self::$suffix . '.css', [], self::$version );
		wp_add_inline_style( 'mdp-revealer', $css . Settings::get_instance()->options['custom_css'] );
	}

	/**
	 * Initialize main variables.
	 *
	 * @since  1.0.0
	 * @access public
	 **/
	public function initialization() {

		/** Plugin version. */
		if ( ! function_exists('get_plugin_data') ) {
			require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		$plugin_data = get_plugin_data( __FILE__ );
		self::$version = $plugin_data['Version'];

		/** Gets the plugin URL (with trailing slash). */
		self::$url = plugin_dir_url( __FILE__ );

		/** Gets the plugin PATH. */
		self::$path = plugin_dir_path( __FILE__ );

		/** Use minified libraries if SCRIPT_DEBUG is turned off. */
		self::$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

		/** Set plugin basename. */
		self::$basename = plugin_basename( __FILE__ );

		/** Initialize plugin settings. */
		Settings::get_instance();

		/** Initialize PluginHelper. */
		PluginHelper::get_instance();

		/** Plugin update mechanism enable only if plugin have Envato ID. */
		$plugin_id = EnvatoItem::get_instance()->get_id();
		if ( (int)$plugin_id > 0 ) {
			PluginUpdater::get_instance();
		}

	}

	/**
	 * Add ajax action to the array of allowed actions
	 * to ensure the DIVI builder modules are loaded for ajax callback.
	 *
	 * @since  1.0.0
	 * @access public
	 **/
	public function divi_builder_load_actions() {
		$actions[] = 'get_page_by_url';

		return $actions;
	}

	/**
	 * Ajax front-end action hook here.
	 *
	 * @since  1.0.0
	 * @access public
	 **/
	public function get_page_by_url() {

		/** Get Target URL. */
		if ( ! isset( $_REQUEST['url'] ) ) {
			return;
		}
		$url = filter_var( $_REQUEST['url'], FILTER_SANITIZE_URL );

		/** Get Post ID. */
		$post_id = url_to_postid( $url );
		if ( ! $post_id ) {
			return;
		}

		$post = get_post( $post_id );

		/** Post Title. */
		$title = $post->post_title;

		/** Post Author. */
		$authorID = $post->post_author;
		$author   = get_the_author_meta( 'display_name', $authorID );

		/** Post Category. */
		$categories = get_the_category( $post_id );
		$category   = '';
		foreach ( $categories as $key => $cat ) {
			$category .= ( count( $categories ) > 1 && count( $categories ) > $key + 1 ) ? $cat->cat_name . ', ' : $cat->cat_name;
		}

		/** Post Comments */
		$comments = $post->comment_count;

		/** Post Excerpt. */
		if ( has_excerpt( $post->ID ) ) {
			$excerpt = get_the_excerpt( $post->ID );
		} else {

			/** Make sure all vc shortcodes are loaded. */
			if ( method_exists( 'WPBMap', 'addAllMappedShortcodes' ) ) {
				/** @noinspection PhpUndefinedClassInspection */
				WPBMap::addAllMappedShortcodes();
			}

			/** Current post Content. */
			$excerpt = apply_filters( 'the_content', $post->post_content );
		}

		$excerpt = wp_trim_words( $excerpt, Settings::get_instance()->options['excerpt_length'], ' ...' );
		$excerpt = strip_tags( $excerpt, '<strong><b>' );
		$excerpt = preg_replace( "/<([a-z][a-z0-9]*)[^>]*?(\/?)>/i", '<$1$2>', $excerpt );

		/** Image. */
		if ( Settings::get_instance()->options['image_position'] == 'without-image' ) {
			$src = '';
		} else {
			$src = get_the_post_thumbnail_url( $post->ID, Settings::get_instance()->options['image_size'] );
		}

		/** Image Position. */
		$image_position_class = '';

		if ( in_array( Settings::get_instance()->options['image_position'], [ 'without-image', 'top', 'bottom', 'left', 'right' ] ) ) {
			$image_position_class = Settings::get_instance()->options['image_position'];

		} elseif ( Settings::get_instance()->options['image_position'] == 'top-bottom' ) {
			$classes              = [ 'top', 'bottom' ];
			$image_position_class = $classes[ array_rand( $classes ) ];

		} elseif ( Settings::get_instance()->options['image_position'] == 'top-left' ) {
			$classes              = [ 'top', 'left' ];
			$image_position_class = $classes[ array_rand( $classes ) ];

		} elseif ( Settings::get_instance()->options['image_position'] == 'top-right' ) {
			$classes              = [ 'top', 'right' ];
			$image_position_class = $classes[ array_rand( $classes ) ];

		} elseif ( Settings::get_instance()->options['image_position'] == 'bottom-left' ) {
			$classes              = [ 'bottom', 'left' ];
			$image_position_class = $classes[ array_rand( $classes ) ];

		} elseif ( Settings::get_instance()->options['image_position'] == 'bottom-right' ) {
			$classes              = [ 'bottom', 'right' ];
			$image_position_class = $classes[ array_rand( $classes ) ];

		} elseif ( Settings::get_instance()->options['image_position'] == 'random' ) {
			$classes              = [ 'top', 'bottom', 'left', 'right' ];
			$image_position_class = $classes[ array_rand( $classes ) ];
		}

		/** Popup content. */
		ob_start();
		?>
        <div class="mdp-revealer-box <?php echo esc_attr( $image_position_class ); ?>">
			<?php if ( $src ) : ?>
                <a href="<?php echo esc_url( $url ); ?>" class="mdp-image">
                    <img src="<?php echo esc_attr( $src ); ?>" alt="<?php echo esc_attr( $title ); ?>"/>
                </a>
			<?php endif; ?>

            <a class="mdp-content" href="<?php echo esc_url( $url ); ?>">

				<?php if ( Settings::get_instance()->options['author_position'] == 'top' || Settings::get_instance()->options['category_position'] == 'top' || Settings::get_instance()->options['comments_position'] == 'top' ) : ?>
                    <div class="mdp-header">
                        <p>
							<?php if ( Settings::get_instance()->options['author_position'] == 'top' ) : ?>
                                <span class="mdp-author"><?php echo esc_attr( $author ); ?></span>
							<?php endif; ?>
							<?php if ( Settings::get_instance()->options['category_position'] == 'top' && count( $categories ) > 0 ) : ?>
                                <span class="mdp-category"><?php echo esc_attr( $category ); ?></span>
							<?php endif; ?>
							<?php if ( Settings::get_instance()->options['comments_position'] == 'top' ) : ?>
                                <span class="mdp-comments"><?php echo esc_attr( $comments ); ?></span>
							<?php endif; ?>
                        </p>
                    </div>
				<?php endif; ?>

				<?php if ( $title ) : ?>
                    <h4><?php esc_html_e( $title ); ?></h4>
				<?php endif; ?>

				<?php if ( $excerpt ) : ?>
                    <div class="mdp-excerpt"><p><?php echo wp_kses_post( $excerpt ); ?></p></div>
				<?php endif; ?>

				<?php if ( Settings::get_instance()->options['author_position'] == 'bottom' || Settings::get_instance()->options['category_position'] == 'bottom' || Settings::get_instance()->options['comments_position'] == 'bottom' ) : ?>
                    <div class="mdp-footer">
                        <p>
							<?php if ( Settings::get_instance()->options['author_position'] == 'bottom' ) : ?>
                                <span class="mdp-author"><?php echo esc_attr( $author ); ?></span>
							<?php endif; ?>
							<?php if ( Settings::get_instance()->options['category_position'] == 'bottom' && count( $categories ) > 0 ) : ?>
                                <span class="mdp-category"><?php echo esc_attr( $category ); ?></span>
							<?php endif; ?>
							<?php if ( Settings::get_instance()->options['comments_position'] == 'bottom' ) : ?>
                                <span class="mdp-comments"><?php echo esc_attr( $comments ); ?></span>
							<?php endif; ?>
                        </p>
                    </div>
				<?php endif; ?>

            </a>
        </div>
		<?php
		$html = ob_get_clean();


		/**
		 * Return Result.
		 * $html escaped above.
		 **/
		echo json_encode( [ 'status' => 1, 'html' => $html ] );
		die();
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since   1.0.0
	 * @return void
	 **/
	public function scripts() {

		/** Checks if plugin should work on this page. */
		if ( AssignmentsTab::get_instance()->display() ) {

			wp_enqueue_script( 'popper', self::$url . 'js/popper.min.js', [], self::$version, true );
			wp_enqueue_script( 'tippy', self::$url . 'js/tippy.min.js', [], self::$version, true );
			wp_enqueue_script( 'mdp-revealer', self::$url . 'js/revealer' . self::$suffix . '.js', ['popper', 'tippy'], self::$version, true );
			wp_localize_script( 'mdp-revealer', 'mdp_revealer',
				/** We use mdp_ prefix to avoid conflicts in JS. */
				[
					'ajaxurl'   => admin_url( 'admin-ajax.php' ),
					'selector'  => Settings::get_instance()->options['selector'], // Selector.
					'theme'     => Settings::get_instance()->options['style'], // Style.
					'animation' => Settings::get_instance()->options['animation'], // Animation.
					'maxWidth'  => ( (int)Settings::get_instance()->options['size'] ) . 'px', // Pop-up Size.
					'arrow'     => ( Settings::get_instance()->options['arrow'] == 'without-arrow' ) ? false : true, // Arrow.
					'arrowType' => Settings::get_instance()->options['arrow'] // Arrow style.
				]
			);

		}

	}

	/**
	 * Loads the Revealer translated strings.
	 *
	 * @since  1.0.0
	 * @access public
	 **/
	public function load_textdomain() {

		load_plugin_textdomain( 'revealer', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

	}

	/**
	 * Return plugin version.
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 **/
	public function get_version() {
		return self::$version;
	}

	/**
	 * Add CSS for admin area.
	 *
	 * @since   1.0.0
	 * @return void
	 **/
	public function admin_styles() {

		/** Add styles only on setting page */
		$screen = get_current_screen();

		if ( $screen->base == "toplevel_page_mdp_revealer_settings" ) {
			wp_enqueue_style( 'merkulov-ui', self::$url . 'css/merkulov-ui' . self::$suffix . '.css', array(), self::$version );
			wp_enqueue_style( 'mdp-revealer-admin', self::$url . 'css/admin' . self::$suffix . '.css', [], self::$version );

        /** Revealer popup on update. */
		} elseif ( 'plugin-install' == $screen->base ) {

			/** Styles only for our plugin. */
			if ( isset( $_GET['plugin'] ) AND $_GET['plugin'] === 'revealer' ) {
				wp_enqueue_style( 'mdp-revealer-plugin-install', self::$url . 'css/plugin-install' . self::$suffix . '.css', [], self::$version );
			}
		}

	}

	/**
	 * Add JS for admin area.
	 *
	 * @since   1.0.0
	 * @return void
	 **/
	public function admin_scripts() {

		/** Add styles only on setting page */
		$screen = get_current_screen();

		if ( $screen->base == "toplevel_page_mdp_revealer_settings" ) {
			wp_enqueue_script( 'merkulov-ui', self::$url . 'js/merkulov-ui' . self::$suffix . '.js', array(), self::$version, true );
			wp_enqueue_script( 'mdp-revealer-admin', self::$url . 'js/admin' . self::$suffix . '.js', [ 'jquery' ], self::$version, true );
		}
	}

	/**
	 * Run when the plugin is activated.
	 *
	 * @static
	 * @since 1.0.0
	 **/
	public static function on_activation() {

		/** Security checks. */
		if ( ! current_user_can( 'activate_plugins' ) ) {
			return;
		}

		$plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
		check_admin_referer( "activate-plugin_{$plugin}" );

		/** Send install Action to our host. */
		Helper::get_instance()->send_action( 'install', 'revealer', self::$version );

	}

	/**
	 * Main Revealer Instance.
	 *
	 * Insures that only one instance of Revealer exists in memory at any one time.
	 *
	 * @static
	 * @since 1.0.0
	 * @return Revealer
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Revealer ) ) {
			self::$instance = new Revealer;
		}

		return self::$instance;
	}

} // End Class Revealer.

/** Run when the plugin is activated. */
register_activation_hook( __FILE__, ['Merkulove\Revealer', 'on_activation'] );

/** Run Revealer class. */
Revealer::get_instance();